from collections import defaultdict
from datetime import datetime as dt
from functools import wraps

import quilt3 as qt


def timeit(func):
    """
    Timing function to time error collection.
    Args:
        func:

    Returns:
    """

    @wraps(func)
    def timed(*args, **kw):
        tstart = dt.now()
        output = func(*args, **kw)
        tend = dt.now()
        print('"{}" took {} to execute.\n'.format(func.__name__, (tend - tstart)))
        return output

    return timed


def construct_args_for_pool(repos, bucket):
    """
    Given a list of repos and bucket, it creates a dict of arguments for process workers
    Args:
        repos: list of repos for workers to work on
        bucket: s3 bucket

    Returns:
        args_for_pool [dict]: list of arguments against a repo for the workers.
    """
    args_for_pool = defaultdict(list)
    for repo in repos:
        print(f'Getting manifest for: {repo}')
        pkg = qt.Package.browse(repo, bucket)
        for key, entry in pkg.walk():
            args_for_pool[repo].append((repo, key, entry.meta))
    return args_for_pool


def list_to_set_in_dict(obj):
    """
    Recursively convert list values inside a dic to sets
    Args:
        obj: Current value being checked

    Returns:
        obj: Dict with lists converted to sets.
    """
    if isinstance(obj, list):
        return {list_to_set_in_dict(v) for v in obj}
    elif isinstance(obj, dict):
        return {k: list_to_set_in_dict(v) for k, v in obj.items()}
    return obj


def lower_and_strip(obj):
    """
    Lower and strip all values in a list
    Args:
        obj:

    Returns:

    """
    for key, val in obj.items():
        if isinstance(val, list):
            obj[key] = [v.lower().strip() for v in val]
        elif isinstance(val, dict):
            obj[key] = lower_and_strip(obj[key])
    return obj
